package com.fs.company.mapper;

import java.util.List;
import java.util.Map;

import com.fs.company.domain.CompanyTagUser;
import com.fs.company.vo.CompanyTagUserVO;
import com.fs.store.param.h5.TagListParam;
import com.fs.store.vo.h5.CompanyUserTagListVO;
import org.apache.ibatis.annotations.Param;

/**
 * companyMapper接口
 *
 * @author fs
 * @date 2025-04-02
 */
public interface CompanyTagUserMapper
{
    /**
     * 查询company
     *
     * @param id companyID
     * @return company
     */
    public CompanyTagUser selectCompanyTagUserById(Long id);

    /**
     * 查询company列表
     *
     * @param companyTagUser company
     * @return company集合
     */
    public List<CompanyTagUser> selectCompanyTagUserList(CompanyTagUser companyTagUser);

    /**
     * 新增company
     *
     * @param companyTagUser company
     * @return 结果
     */
    public int insertCompanyTagUser(CompanyTagUser companyTagUser);

    /**
     * 修改company
     *
     * @param companyTagUser company
     * @return 结果
     */
    public int updateCompanyTagUser(CompanyTagUser companyTagUser);

    /**
     * 删除company
     *
     * @param id companyID
     * @return 结果
     */
    public int deleteCompanyTagUserById(Long id);

    /**
     * 批量删除company
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteCompanyTagUserByIds(Long[] ids);

    List<CompanyUserTagListVO> getTagList(@Param("param") TagListParam param, @Param("keywords") String[] keywords, @Param("userIds") List<Long> userIds);

    /**
     * 根据条件查询标签下用户
     * @param params    条件
     * @return  list
     */
    List<CompanyTagUserVO> selectUserListByMap(@Param("params") Map<String, Object> params);

    /**
     * 删除用户标签
     * @param params    条件
     */
    void deleteCompanyTagUserByMap(@Param("params") Map<String, Object> params);
}
